///////////////////////////////////////////////////////////////////////////
//  uCore.h
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//  Description
//
//  This class is expected to work as a helper to work with big data set
//  even if the have an sparse acces.
//
//  The use wold be some thin like this:
//  FileMap       gameFile;
//  const char*   pGameROM;
//  if(gameFile.Create("myGame.rom", 0, -1, 8*1) ) //  2MB cache, -1 means whole file
//    pGameROM = gameFile.GetVirtualAddr();
//  else
//    ReportError("Unable to map file!");
//
//  Now you can acces freely to pGameROM, because its addresed to a "virtual" direction
//  mapped to somo zones in the internal cache memory.
//  This way if you detect a GP2 with more ram you just increase the case size and the loading
//  times will decrease :-)
//
//
//  About the implementation;
//  I assume 64kb pages, I means we will load dat on 64kb basis
//
//  Internally all the memory from GetVirtualAddr() to  GetVirtualAddr()+GetVirtualSize()
//  will be read and write protected, we ignore all write exceptions, nobody sould write there anyway
//
//  On a read attempt we will access to the VirtualPage structure at 
//  m_aPages[(addres-m_pVirtualAddr)>>16], 
//  if it has a null m_pReallAddr, we will look for an unused
//  real memory page  and unprotecct read acces to that virtual address,
//  if we dont find any unused real memory page we will discard one from the m_qDiscardOrder,
//
//  Ince we have a real memory page load the content of the file there, map the virtual addr to thar addr
//  and insert it to the m_qDiscardOrder que.
//
//
///////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////
class FileMap
{
public:
  bool        Create(char* szFileName, int iFileOffset, int iFileSize, int iCachePages);
  void        Destroy();
  const void* GetVirtualAddr() const;
  int         GetVirtualSize() const;

protected:
  struct  VirtualPage
  {
    void* m_pRealAddr;
    int   m_iLastUsed;
  };
  
  VirtualPage* LoadPage();
  void         DiscardPage();
  VirtualPage* FindFreePage();

  int           m_iCachePages;    //  iCacheSize = numPage*65536;
  void*         m_pCacheAddr;
  void*         m_pVirtualAddr;

  VirtualPage*  m_aPages;         //  iCachePages

  void**        m_aFreeRealPages;
  VirtualPage*  m_iFreeRealPaegs;

  PriQue<VirtualPage*>  m_qDiscardOrder;        
};



